from typing import Tuple

import gin
from transformers import AutoModelForSequenceClassification, AlbertForSequenceClassification, \
    BertForSequenceClassification, AutoTokenizer, AutoConfig, AutoModel, BertTokenizer
import numpy as np
import logging
import os
from typing import Dict, Type, Callable, List
import transformers
import torch
from torch import nn
from torch.optim import Optimizer
from torch.utils.data import DataLoader
from tqdm.autonotebook import tqdm, trange
from sentence_transformers import SentenceTransformer


# from .. import SentenceTransformer
# from ..evaluation import SentenceEvaluator

class AutoBertForSequenceClassification(nn.Module):
    def __init__(
            self,
            pretrained: str,
            config,
            mode: str = 'cls',
            task: str = 'classification'
    ) -> None:
        super(AutoBertForSequenceClassification, self).__init__()
        self._pretrained = pretrained
        self._mode = mode
        self._task = task
        self.config = config

        # self._config = AutoConfig.from_pretrained(self._pretrained)
        self._model = AutoModel.from_pretrained(self._pretrained, config=self.config)
        if self._task == 'ranking':
            self._dense = nn.Linear(self.config.hidden_size, 1)
        elif self._task == 'classification':
            self._dense = nn.Linear(self.config.hidden_size, self.config.num_labels)
        else:
            raise ValueError('Task must be `ranking` or `classification`.')

    def forward(self, input_ids: torch.Tensor, attention_mask: torch.Tensor = None,
                token_type_ids: torch.Tensor = None) -> Tuple[torch.Tensor, torch.Tensor]:
        if token_type_ids is not None:
            output = self._model(input_ids, attention_mask=attention_mask, token_type_ids=token_type_ids)
        else:
            output = self._model(input_ids, attention_mask=attention_mask)
        if self._mode == 'cls':
            score = self._dense(output[0][:, 0, :]).squeeze(-1)
        elif self._mode == 'pooling':
            score = self._dense(output[1]).squeeze(-1)
        else:
            raise ValueError('Mode must be `cls` or `pooling`.')
        return score, output[0][:, 0, :]


class SentenceEncoder():
    def __init__(self, model_name, tokenzier, config, num_labels: int = None, max_length: int = None, device: str = None):
        """
        A CrossEncoder takes exactly two sentences / texts as input and either predicts
        a score or label for this sentence pair. It can for example predict the similarity of the sentence pair
        on a scale of 0 ... 1.

        It does not yield a sentence embedding and does not work for individually sentences.

        :param model: Any model from Huggingface that can be loaded with AutoModel
        :param tokenzier: Any tokenizer from Huggingface that can be loaded with AutoTokenizer
        :param config: Any model config from Huggingface Models Repository that can be loaded with AutoConfig.
        :param num_labels: Number of labels of the classifier. If 1, the CrossEncoder is a regression model that outputs a continous score 0...1. If > 1, it output several scores that can be soft-maxed to get probability scores for the different classes.
        :param max_length: Max length for input sequences. Longer sequences will be truncated. If None, max length of the model will be used
        :param device: Device that should be used for the model. If None, it will use CUDA if available.
        """

        self.config = config
        self.tokenizer = tokenzier

        classifier_trained = True
        if self.config.architectures is not None:
            classifier_trained = any([arch.endswith('ForSequenceClassification') for arch in self.config.architectures])

        if num_labels is None and not classifier_trained:
            num_labels = 1

        if num_labels is not None:
            self.config.num_labels = num_labels
        
        self.model = AutoModelForSequenceClassification.from_pretrained(model_name, config=self.config)
        self.max_length = max_length

        if device is None:
            device = "cuda" if torch.cuda.is_available() else "cpu"
            logging.info("Use pytorch device: {}".format(device))

        self._target_device = torch.device(device)

    def smart_batching_collate(self, batch):
        texts = [[] for _ in range(len(batch[0].texts))]
        labels = []

        for example in batch:
            for idx, text in enumerate(example.texts):
                texts[idx].append(text)

            labels.append(example.label)

        tokenized = self.tokenizer(*texts, padding=True, truncation='longest_first', return_tensors="pt",
                                   max_length=self.max_length)
        labels = torch.tensor(labels, dtype=torch.float if self.config.num_labels == 1 else torch.long).to(
            self._target_device)

        for name in tokenized:
            tokenized[name] = tokenized[name].to(self._target_device)

        return tokenized, labels

    def smart_batching_collate_text_only(self, batch):
        texts = [[] for _ in range(len(batch[0]))]

        for example in batch:
            for idx, text in enumerate(example):
                texts[idx].append(text)

        tokenized = self.tokenizer(*texts, padding=True, truncation='longest_first', return_tensors="pt",
                                   max_length=self.max_length)

        for name in tokenized:
            tokenized[name] = tokenized[name].to(self._target_device)

        return tokenized

    
    def fit(self,
            train_dataloader: DataLoader,
            evaluator=None,
            epochs: int = 1,
            loss_fct=None,
            acitvation_fct=nn.Identity(),
            scheduler: str = 'WarmupLinear',
            warmup_steps: int = 10000,
            optimizer_class: Type[Optimizer] = transformers.AdamW,
            optimizer_params: Dict[str, object] = {'lr': 2e-5, 'eps': 1e-6, 'correct_bias': False},
            weight_decay: float = 0.01,
            evaluation_steps: int = 0,
            output_path: str = None,
            save_best_model: bool = True,
            max_grad_norm: float = 1,
            use_amp: bool = False,
            callback: Callable[[float, int, int], None] = None,
            ):
        #todo: 添加cls分层器
        """
        Train the model with the given training objective
        Each training objective is sampled in turn for one batch.
        We sample only as many batches from each objective as there are in the smallest one
        to make sure of equal training with each dataset.

        :param train_dataloader: DataLoader with training InputExamples
        :param evaluator: An evaluator (sentence_transformers.evaluation) evaluates the model performance during training on held-out dev data. It is used to determine the best model that is saved to disc.
        :param epochs: Number of epochs for training
        :param loss_fct: Which loss function to use for training. If None, will use nn.BCEWithLogitsLoss() if self.config.num_labels == 1 else nn.CrossEntropyLoss()
        :param acitvation_fct: Activation function applied on top of logits output of model.
        :param scheduler: Learning rate scheduler. Available schedulers: constantlr, warmupconstant, warmuplinear, warmupcosine, warmupcosinewithhardrestarts
        :param warmup_steps: Behavior depends on the scheduler. For WarmupLinear (default), the learning rate is increased from o up to the maximal learning rate. After these many training steps, the learning rate is decreased linearly back to zero.
        :param optimizer_class: Optimizer
        :param optimizer_params: Optimizer parameters
        :param weight_decay: Weight decay for model parameters
        :param evaluation_steps: If > 0, evaluate the model using evaluator after each number of training steps
        :param output_path: Storage path for the model and evaluation files
        :param save_best_model: If true, the best model (according to evaluator) is stored at output_path
        :param max_grad_norm: Used for gradient normalization.
        :param use_amp: Use Automatic Mixed Precision (AMP). Only for Pytorch >= 1.6.0
        :param callback: Callback function that is invoked after each evaluation.
                It must accept the following three parameters in this order:
                `score`, `epoch`, `steps`
        """
        train_dataloader.collate_fn = self.smart_batching_collate

        if use_amp:
            from torch.cuda.amp import autocast
            scaler = torch.cuda.amp.GradScaler()

        self.model.to(self._target_device)

        if output_path is not None:
            os.makedirs(output_path, exist_ok=True)

        self.best_score = -9999999
        self.best_prediction = None
        self.predict_time = None
        num_train_steps = int(len(train_dataloader) * epochs)

        # Prepare optimizers
        param_optimizer = list(self.model.named_parameters())

        no_decay = ['bias', 'LayerNorm.bias', 'LayerNorm.weight']
        optimizer_grouped_parameters = [
            {'params': [p for n, p in param_optimizer if not any(nd in n for nd in no_decay)],
             'weight_decay': weight_decay},
            {'params': [p for n, p in param_optimizer if any(nd in n for nd in no_decay)], 'weight_decay': 0.0}
        ]

        optimizer = optimizer_class(optimizer_grouped_parameters, **optimizer_params)

        if isinstance(scheduler, str):
            scheduler = SentenceTransformer._get_scheduler(optimizer, scheduler=scheduler, warmup_steps=warmup_steps,
                                                           t_total=num_train_steps)

        if loss_fct is None:
            loss_fct = nn.BCEWithLogitsLoss() if self.config.num_labels == 1 else nn.CrossEntropyLoss()

        print("check self.config.num_labels", self.config.num_labels)
        skip_scheduler = False
        for epoch in trange(epochs, desc="Epoch"):
            training_steps = 0
            # self.model.zero_grad()
            self.model.train()

            for features, labels in tqdm(train_dataloader, desc="Iteration", smoothing=0.05):
                self.model.train()
                if use_amp:
                    with autocast():
                        model_predictions = self.model(**features, return_dict=True)
                        logits = acitvation_fct(model_predictions.logits)
                        print('logits,',logits)
                        print("num_labels", self.config.num_labels)
                        if self.config.num_labels == 1:
                            logits = logits.view(-1)
                        loss_value = loss_fct(logits, labels)

                    scale_before_step = scaler.get_scale()
                    scaler.scale(loss_value).backward()
                    scaler.unscale_(optimizer)
                    torch.nn.utils.clip_grad_norm_(self.model.parameters(), max_grad_norm)
                    scaler.step(optimizer)
                    scaler.update()

                    skip_scheduler = scaler.get_scale() != scale_before_step
                else:
                    features.update({"labels": labels})
                    model_predictions = self.model(**features, return_dict=True)

                    logits = acitvation_fct(model_predictions.logits)
                    if self.config.num_labels == 1:
                        logits = logits.view(-1)

                    loss_value = loss_fct(logits, labels)
                    # print(loss_value.item())
                    loss_value.backward()
                    torch.nn.utils.clip_grad_norm_(self.model.parameters(), max_grad_norm)
                    optimizer.step()

                optimizer.zero_grad()

                if not skip_scheduler:
                    scheduler.step()

                training_steps += 1

                if evaluator is not None and evaluation_steps > 0 and training_steps % evaluation_steps == 0:
                    self._eval_during_training(evaluator, output_path, save_best_model, epoch, training_steps, callback)

                    # self.model.zero_grad()
                    self.model.train()

            if evaluator is not None:
                self._eval_during_training(evaluator, output_path, save_best_model, epoch, -1, callback)
                self.model.train()

    def predict(self, sentences: List[List[str]],
                batch_size: int = 32,
                show_progress_bar: bool = None,
                num_workers: int = 0,
                activation_fct=None,
                apply_softmax=False,
                convert_to_numpy: bool = True,
                convert_to_tensor: bool = False
                ):
        """
        Performs predicts with the CrossEncoder on the given sentence pairs.

        :param sentences: A list of sentence pairs [[Sent1, Sent2], [Sent3, Sent4]]
        :param batch_size: Batch size for encoding
        :param show_progress_bar: Output progress bar
        :param num_workers: Number of workers for tokenization
        :param activation_fct: Activation function applied on the logits output of the CrossEncoder. If None, nn.Sigmoid() will be used if num_labels=1, else nn.Identity
        :param convert_to_numpy: Convert the output to a numpy matrix.
        :param apply_softmax: If there are more than 2 dimensions and apply_softmax=True, applies softmax on the logits output
        :param convert_to_tensor:  Conver the output to a tensor.
        :return: Predictions for the passed sentence pairs
        """
        input_was_string = False
        if isinstance(sentences[0], str):  # Cast an individual sentence to a list with length 1
            sentences = [sentences]
            input_was_string = True

        inp_dataloader = DataLoader(sentences, batch_size=batch_size, collate_fn=self.smart_batching_collate_text_only,
                                    num_workers=num_workers, shuffle=False)

        if show_progress_bar is None:
            show_progress_bar = (
                        logging.getLogger().getEffectiveLevel() == logging.INFO or logging.getLogger().getEffectiveLevel() == logging.DEBUG)

        iterator = inp_dataloader
        if show_progress_bar:
            iterator = tqdm(inp_dataloader, desc="Batches")

        if activation_fct is None:
            activation_fct = nn.Sigmoid() if self.config.num_labels == 1 else nn.Identity()

        pred_scores = []
        self.model.eval()
        self.model.to(self._target_device)
        with torch.no_grad():
            for features in iterator:
                model_predictions = self.model(**features, return_dict=True)
                logits = activation_fct(model_predictions.logits)

                if apply_softmax and len(logits[0]) > 1:
                    logits = torch.nn.functional.softmax(logits, dim=1)
                pred_scores.extend(logits)

        if self.config.num_labels == 1:
            pred_scores = [score[0] for score in pred_scores]

        if convert_to_tensor:
            pred_scores = torch.stack(pred_scores)
        elif convert_to_numpy:
            pred_scores = np.asarray([score.cpu().detach().numpy() for score in pred_scores])

        if input_was_string:
            pred_scores = pred_scores[0]

        return pred_scores

    def _eval_during_training(self, evaluator, output_path, save_best_model, epoch, steps, callback):
        """Runs evaluation during the training"""
        if evaluator is not None:
            score = evaluator(self, output_path=output_path, epoch=epoch, steps=steps)
            if callback is not None:
                callback(score, epoch, steps)
            if score > self.best_score:
                self.best_score = score
                # self.best_prediction = pred
                # self.predict_time = pred_time
                if save_best_model:
                    self.save(output_path)

    def save(self, path):
        """
        Saves all model and tokenizer to path
        """
        if path is None:
            return

        logging.info("Save model to {}".format(path))
        self.model.save_pretrained(path)
        self.tokenizer.save_pretrained(path)

    def save_pretrained(self, path):
        """
        Same function as save
        """
        return self.save(path)



from transformers import BertTokenizer, AutoModel, Text2TextGenerationPipeline
tokenizer = BertTokenizer.from_pretrained("bert-base-uncased")
model = AutoModel.from_pretrained("bert-base-uncased")
tokenizer.save_pretrained(save_directory="D:\code\codalab\starting_kit\sample_code_submission\\bert-base-uncased")
model.save_pretrained(save_directory="D:\code\codalab\starting_kit\sample_code_submission\\bert-base-uncased")





# -*- coding: utf-8 -*-

import re
import numpy as np

from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.svm import LinearSVC

import os

os.system('pip install jieba')
os.system("pip install transformers==4.9.1")
os.system("pip install gin-config==0.3.0")
import jieba
import gin

from hf_gp import HFOp

MAX_VOCAB_SIZE = 20000


# code form https://towardsdatascience.com/multi-class-text-classification-with-lstm-1590bee1bd17
def clean_en_text(dat):
    REPLACE_BY_SPACE_RE = re.compile('["/(){}\[\]\|@,;]')
    BAD_SYMBOLS_RE = re.compile('[^0-9a-zA-Z #+_]')
    
    ret = []
    for line in dat:
        # text = text.lower() # lowercase text
        line = REPLACE_BY_SPACE_RE.sub(' ', line)
        line = BAD_SYMBOLS_RE.sub('', line)
        line = line.strip()
        ret.append(line)
    return ret


def clean_zh_text(dat):
    REPLACE_BY_SPACE_RE = re.compile('[“”【】/（）：！～「」、|，；。"/(){}\[\]\|@,\.;]')
    
    ret = []
    for line in dat:
        line = REPLACE_BY_SPACE_RE.sub(' ', line)
        line = line.strip()
        ret.append(line)
    return ret


def _tokenize_chinese_words(text):
    return ' '.join(jieba.cut(text, cut_all=False))


def vectorize_data(x_train, x_val=None):
    vectorizer = TfidfVectorizer(ngram_range=(1, 1), max_features=MAX_VOCAB_SIZE)
    if x_val:
        full_text = x_train + x_val
    else:
        full_text = x_train
    vectorizer.fit(full_text)
    train_vectorized = vectorizer.transform(x_train)
    if x_val:
        val_vectorized = vectorizer.transform(x_val)
        return train_vectorized, val_vectorized, vectorizer
    return train_vectorized, vectorizer


# onhot encode to category
def ohe2cat(label):
    return np.argmax(label, axis=1)


class model(object):
    """ model of SVM baseline """
    
    def __init__(self, metadata):
        """ Initialization for model
        :param metadata: a dict formed like:
            {"class_num": 10,
             "language": ZH,
             "num_train_instances": 10000,
             "num_test_instances": 1000,
             "time_budget": 300}
        """
        self.done_training = False
        self.metadata = metadata
        self.train_output_path = './'
        self.test_input_path = './'
        self.model = LinearSVC(random_state=0, tol=1e-5, max_iter=500)
        code_dir = os.path.dirname(__file__)
        if (os.path.exists(os.path.join(code_dir, 'hf_gp.gin'))):
            gin.parse_config_file(os.path.join(code_dir, 'hf_gp.gin'))
        self.op = HFOp(metadata)
        print("load HF Op!")
        
        self.tokenizer = TfidfVectorizer()
    
    def fit(self, x_train, y_train, remaining_time_budget=None):
        
        """model training on train_dataset.
        
        :param train_dataset: tuple, (x_train, y_train)
            x_train: list of str, input training sentences.
            y_train: A `numpy.ndarray` matrix of shape (sample_count, class_num).
                     here `sample_count` is the number of examples in this dataset as train
                     set and `class_num` is the same as the class_num in metadata. The
                     values should be binary.
        :param remaining_time_budget:
        """
        if self.done_training:
            return
        # tokenize Chinese words
        if self.metadata['language'] == 'ZH':
            print(x_train.shape)
            
            x_train = clean_zh_text(x_train)
            x_train = list(map(_tokenize_chinese_words, x_train))
            print(x_train)
        else:
            x_train = clean_en_text(x_train)
        
        # x_train, self.tokenizer = vectorize_data(x_train)
        # model = LinearSVC(random_state=0, tol=1e-5, max_iter=500)
        print(str(type(x_train)) + " " + str(y_train.shape))
        
        kwargs = {"label_num": self.metadata["class_num"],
                  "max_length": 128}
       
        self.op.fit(x_train, y_train, remaining_time_budget,  **kwargs)
        self.model = self.op
        # with open(self.train_output_path + 'model.pickle', 'wb') as handle:
        #     pickle.dump(model, handle, protocol=pickle.HIGHEST_PROTOCOL)
        
        # with open(self.train_output_path + 'tokenizer.pickle', 'wb') as handle:
        #     pickle.dump(tokenizer, handle, protocol=pickle.HIGHEST_PROTOCOL)
        
        self.done_training=True
    
    def predict(self, x_test, remaining_time_budget=None):
        """
        :param x_test: list of str, input test sentences.
        :param remaining_time_budget:
        :return: A `numpy.ndarray` matrix of shape (sample_count, class_num).
                 here `sample_count` is the number of examples in this dataset as test
                 set and `class_num` is the same as the class_num in metadata. The
                 values should be binary or in the interval [0,1].
        """
        # with open(self.test_input_path + 'model.pickle', 'rb') as handle:
        #     model = pickle.load(handle, encoding='iso-8859-1')
        # with open(self.test_input_path + 'tokenizer.pickle', 'rb') as handle:
        #     tokenizer = pickle.load(handle, encoding='iso-8859-1')
        
        train_num, test_num = self.metadata['train_num'], self.metadata['test_num']
        class_num = self.metadata['class_num']
        
        # tokenizing Chinese words
        if self.metadata['language'] == 'ZH':
            x_test = clean_zh_text(x_test)
            x_test = list(map(_tokenize_chinese_words, x_test))
        else:
            x_test = clean_en_text(x_test)

        result = self.model.predict(x_test)
        print(result)
        return result
        # return y_test
    
    def save(self, path="./"):
        '''
        Save a trained model.
        '''
        pass
    
    def load(self, path="./"):
        '''
        Load a trained model.
        '''
        pass



# -*- coding: utf-8 -*-
# @Date    : 2021/9/18 19:53
# @Author  : stellahong (stellahong@fuzhi.ai)
# @Desc    :
class Pipeline():
    """
    Input -> Tokenization -> Model Inference -> Post-Processing (task dependent) -> Output
    """
    def __init__(self):
        pass
    
    def train(self):
        pass
    
    def evaluate(self):
        pass
    
    def predict(self):
        pass



import os

os.environ["CUDA_VISIBLE_DEVICES"] = "0"
import gin
import torch
import numpy as np

from sklearn.model_selection import train_test_split
from transformers import AutoModelForSequenceClassification, AutoModelForQuestionAnswering, \
    AutoModelForTokenClassification
from torch.utils.data.dataloader import DataLoader

from hfgp.base.hf_operator import BaseTrainer
from hfgp.evaluator.sentence_transformer_evaluator import CEAucEvaluator
from hfgp.conf.config import MODEL_SAVE_PATH, MODEL_HUB_PATH

from hfgp.utils.data_utils import convert2clssample
from hfgp.utils.download_utils import download

@gin.configurable
class HFTaskModel():
    # Constructor parameters become configurable.
    def __init__(self,
                 task='',
                 model_name='',
                 backbone=None):
        self.task = task
        self.model_name = model_name
        self.backbone = backbone
        self.__load__()
    
    def __load__(self):
        """
        根据定义的任务类型加载HF模型，目前只支持文本分类，问答匹配，文本序列
        :return:
        """
        if not os.path.exists(MODEL_HUB_PATH + os.sep + self.model_name):
            try:
                download(MODEL_HUB_PATH, self.model_name)
            except:
                raise ValueError("Download Error!")
        if self.task == "text_classification":
            model = AutoModelForSequenceClassification.from_pretrained(MODEL_HUB_PATH + os.sep + self.model_name)
        elif self.task == "token_classification":
            model = AutoModelForTokenClassification.from_pretrained(MODEL_HUB_PATH + os.sep + self.model_name)
        elif self.task == "question_and_answer":
            model = AutoModelForQuestionAnswering.from_pretrained(MODEL_HUB_PATH + os.sep + self.model_name)
        else:
            model = None
            raise NotImplementedError
        self.backbone.load(model, self.model_name)


@gin.configurable
class HFOp(BaseTrainer):
    def __init__(self, meta_data, hf_model):
        """
        HFGP类，实现统一HF模型接入
        :param metadata: a dict formed like:
            {"class_num": 10,
             "language": ZH,
             "num_train_instances": 10000,
             "num_test_instances": 1000,
             "time_budget": 300}
 
        """
        super().__init__()
        self.meta_data = meta_data
        self.hf_model = hf_model
        self.train_dataloader = None
        self.eval_dataloader = None
        self.best_score = 0.0
        self.output_path = MODEL_SAVE_PATH  # 训练模型存储文件夹路径
    
    def fit(self, x_train, y_train, remaining_time_budget=None, **kwargs):
        y_train = y_train.astype(int)
        
        num_class = kwargs["label_num"]
        max_length = kwargs["max_length"]
    
        print(y_train.shape)
        self._init_data_loader(x_train, y_train, num_class=num_class)
        self.hf_model.backbone.build(num_class, max_length)
        
        # fit
        self.hf_model.backbone.fit(train_dataloader=self.train_dataloader,
                                   evaluator=self.evaluator,
                                   output_path=self.output_path)
        
        print("best val auc is {}".format(self.hf_model.backbone.best_score))
        
        self.best_score = self.hf_model.backbone.best_score
        best_predictions = self.predict(self.evaluator.sentence_pairs)
        print("best_predictions", best_predictions)
        return best_predictions
    
    def predict(self, x_test, remaining_time_budget=None):
        return self.hf_model.backbone.predict(x_test)
    
    def save(self, path="./"):
        pass
    
    def load_weights(self, checkpoint):
        """
        实际加载权重，权重更新到GPU，赋值到实际attr
        :param checkpoint:
        :return:
        """
        checkpoint_model = os.path.join(checkpoint, "pytorch_model.bin")
        device = torch.device(
            "cuda" if torch.cuda.is_available() else "cpu")
        
        model_state_dict = torch.load(checkpoint_model, map_location=device)
        self.hf_model.model.load_state_dict(model_state_dict)
        self.hf_model.tokenizer.from_pretrained(checkpoint)
    
    def _init_data_loader(self, x_train, y_train, num_class):
        train_x, valid_x, train_y, valid_y = train_test_split(x_train, y_train, test_size=0.2, random_state=42)
        train_data_y = np.argmax(train_y, axis=1)
        train_samples = convert2clssample(train_x, train_data_y)
        self.train_dataloader = DataLoader(train_samples, shuffle=True,
                                           batch_size=self.hf_model.backbone.train_batch_size)
        dev_samples = convert2clssample(valid_x, valid_y)
        self.evaluator = CEAucEvaluator.from_input_examples(dev_samples, name='dev', num_class=num_class)
    
    def load(self, path="./"):
        """
        加载训练好的模型权重
        :param path: 待加载的模型权重文件夹路径
        :return:
        """
        self.load_weights(checkpoint=path)


if __name__ == "__main__":
    import os
    
    code_dir = os.path.dirname(__file__)
    meta_data = {"class_num": 10,
                 "language": 'ZH',
                 "num_train_instances": 10000,
                 "num_test_instances": 1000,
                 "time_budget": 300}
    print(code_dir)
    if (os.path.exists(os.path.join(code_dir, 'hf_gp.gin'))):
        gin.parse_config_file(os.path.join(code_dir, 'hf_gp.gin'))
    
    model = HFOp(meta_data)
    print(model.hf_model.backbone.model_name)



# -*- coding: utf-8 -*-
# @Date    : 2020/12/16 19:52
# @Author  : stellahong (stellahong@fuzhi.ai)
# @Desc    :
import logging
import os

import torch
import numpy as np
from tqdm import tqdm
import csv
from typing import List
from sentence_transformers import InputExample
from sklearn.metrics import roc_auc_score
from torch.utils.data.dataloader import DataLoader


class CEAucEvaluator():
    """
    This evaluator can be used with the CrossEncoder class.

    It is designed for CrossEncoders with 2 or more outputs. It measure the
    accuracy of the predict class vs. the gold labels.
    """
    
    def __init__(self, sentence_pairs: List[List[str]], labels: List[int], name: str = '', num_class: int = 2):
        self.sentence_pairs = sentence_pairs
        self.labels = labels
        self.name = name
        
        self.csv_file = "CEAucEvaluator" + ("_" + name if name else '') + "_results.csv"
        self.csv_headers = ["epoch", "steps", "Accuracy"]
    
    @classmethod
    def from_input_examples(cls, examples: List[InputExample], **kwargs):
        sentence_pairs = []
        labels = []
        num_class = kwargs["num_class"]
        for example in examples:
            sentence_pairs.append(example.texts)
            labels.append(example.label)
        labels = np.asarray(labels).reshape(-1, num_class)
        return cls(sentence_pairs, labels, **kwargs)
    
    def __call__(self, model, output_path: str = None, epoch: int = -1, steps: int = -1) -> float:
        if epoch != -1:
            if steps == -1:
                out_txt = " after epoch {}:".format(epoch)
            else:
                out_txt = " in epoch {} after {} steps:".format(epoch, steps)
        else:
            out_txt = ":"
        
        logging.info("CEAucEvaluator: Evaluating the model on " + self.name + " dataset" + out_txt)
        pred_scores = model.predict(self.sentence_pairs, convert_to_numpy=True, apply_softmax=True,
                                    show_progress_bar=True)
        
        
        auc = roc_auc_score(self.labels, pred_scores, average='micro')
        
        logging.info("Auc: {:.2f}".format(auc * 1))
        
        if output_path is not None:
            csv_path = os.path.join(output_path, self.csv_file)
            
            output_file_exists = os.path.isfile(csv_path)
            with open(csv_path, mode="a" if output_file_exists else 'w', encoding="utf-8") as f:
                writer = csv.writer(f)
                if not output_file_exists:
                    writer.writerow(self.csv_headers)

                writer.writerow([epoch, steps, auc])
        
        return auc



# -*- coding: utf-8 -*-
# @Date    : 2021/9/16 15:18
# @Author  : stellahong (stellahong@fuzhi.ai)
# @Desc    :
from sentence_transformers import InputExample

def convert2clssample(x, y):
    """
    给定输入样本array, 转化成对应分类样本数据
    :return:
    """
    samples = []
    for x, y in zip(x, y):
        samples.append(InputExample(texts=[" ".join(x.split(" "))[:]], label=y))
    return samples



from distutils.core import setup
from setuptools import setup, find_packages

 
setup(name = "hfgp",
    version = "0.1.1",
    description = "Huggingface GP 接入模块",
    author = "stellahong",
    author_email = "stellahong@fuzhi.a",
    #Name the folder where your packages live:
    #(If you have other packages (dirs) or modules (py files) then
    #put them into the package directory - they will be found recursively.)
    packages = find_packages(),
    #exclude_package_data = {'docs':['1.txt']},  #排除文件
    install_requires = [
        'gin-config==0.3.0',
        #'sentence-transformers==0.4.1.2'
        'sentence-transformers==0.4.1.2',
	'transformers==4.9.1'
    ],
    long_description = """Really long text here."""   
)



# -*- coding: utf-8 -*-
# @Date    : 2020/4/7 11:01
# @Author  : stellahong (stellahong@fuzhi.ai)
# @Desc    :
import os
import sys
import numpy as np
import traceback
import psutil


def show_layer_info(layer_name, layer_out):
    print('[layer]: %s\t[shape]: %s \n%s' % (layer_name, str(layer_out.get_shape().as_list()), show_memory_use()))


def show_memory_use():
    used_memory_percent = psutil.virtual_memory().percent
    strinfo = '{}% memory has been used'.format(used_memory_percent)
    return strinfo


def import_class(import_str):
    mod_str, _sep, class_str = import_str.rpartition('.')
    __import__(mod_str)
    try:
        return getattr(sys.modules[mod_str], class_str)
    except AttributeError:
        raise ImportError('Class %s cannot be found (%s)' %
                          (class_str,
                           traceback.format_exception(*sys.exc_info())))


def import_object(import_str, *args, **kwargs):
    return import_class(import_str)(*args, **kwargs)


def import_module(import_str):
    __import__(import_str)
    return sys.modules[import_str]

def color_msg(msg, color="red"):
    if color == "red":
        return '\033[31m{}\033[0m'.format(msg)

    elif color == "blue":
        return '\033[34m{}\033[0m'.format(msg)

    elif color == "yellow":
        return '\033[33m{}\033[0m'.format(msg)

    elif color == "green":
        return '\033[36m{}\033[0m'.format(msg)


# onhot encode to category
def ohe2cat(label):
    return np.argmax(label, axis=1)

def cat2oht(y, num_class):
    array = np.zeros((y.shape[0], num_class))
    for id, idx in enumerate(y):
        array[id][idx - 1] = 1
    return array
print(cat2oht(np.array([1,0,2,1,0,1]),3))


import os

import gin

from transformers import AutoConfig, AutoTokenizer

from hfgp.conf.config import MODEL_HUB_PATH
from hfgp.base.cls_model_op import SentenceEncoder


@gin.configurable
class BaseTrainer():
    """
    预训模型训练基类
    预训练模型算子基类，加载Huggingface Models Repository中的模型
    定义对应具体模型可调参数
    """
    
    def __init__(self,
                 lr=2e-5,
                 epoch=5,
                 warmup_steps=1000,
                 weight_decay=0.01,
                 train_batch_size=16,
                 max_length=128
                 ):
        self.epoch = epoch
        self.lr = lr
        self.warmup_steps = warmup_steps
        self.weight_decay = weight_decay
        self.train_batch_size = train_batch_size
        self.max_length = max_length
        
        self.model_hub_path = MODEL_HUB_PATH  # 预训练模型加载文件夹路径
        self.best_score = -9999999
    
    def load(self, task_model, model_name):
        self.model_name = model_name
        self.config = AutoConfig.from_pretrained(MODEL_HUB_PATH + os.sep + model_name)
        self.tokenizer = AutoTokenizer.from_pretrained(MODEL_HUB_PATH + os.sep + model_name)
        self.ptm = MODEL_HUB_PATH + os.sep + model_name  # ptm: pre-train-model
    
    def build(self, num_classes, max_length, device=None):
        self.model = SentenceEncoder(self.ptm,
                                     self.tokenizer,
                                     self.config,
                                     num_labels=num_classes,
                                     max_length=max_length,
                                     device=device)
    
    def fit(self, train_dataloader, evaluator, output_path):
        self.model.fit(train_dataloader=train_dataloader,
                       evaluator=evaluator,
                       epochs=self.epoch,
                       evaluation_steps=100,
                       optimizer_params={"lr": self.lr},
                       callback=self.show_score_callback,
                       save_best_model=True,
                       output_path=output_path)
        self.best_score = self.model.best_score
        
    def predict(self, test_data):
        sentneces = [[" ".join(x[0].split(" "))[:]] for x in test_data]
        predictions = self.model.predict(sentences=sentneces, batch_size=32, convert_to_numpy=True, apply_softmax=True)
        return predictions
    
    @staticmethod
    def show_score_callback(score, epoch, steps):
        print(score, epoch, steps)



# -*- coding: utf-8 -*-
# @Date    : 2021/9/27 16:23
# @Author  : stellahong (stellahong@fuzhi.ai)
# @Desc    :

import os

def download(MODEL_HUB_PATH, model_name):
    model_dir = os.path.join(MODEL_HUB_PATH, model_name)
    os.makedirs(model_dir)
    os.system(f"wget -P {model_dir} 192.168.50.198/{model_name}/config.json")
    os.system(f"wget -P {model_dir} 192.168.50.198/{model_name}/tokenizer.json")
    os.system(f"wget -P {model_dir} 192.168.50.198/{model_name}/tokenizer_config.json")
    os.system(f"wget -P {model_dir} 192.168.50.198/{model_name}/vocab.txt")
    os.system(f"wget -P {model_dir} 192.168.50.198/{model_name}/pytorch_model.bin")